/************************************************************************/
/* calls.c: This file contains routines for parsing configurations 	*/
/*          files, parsing distinguished names, and formulating 	*/
/*	    friendly names.						*/
/*                                                                      */
/* Modification Log:                                                    */
/*                                                                      */
/* 91/05/21     Sze-Ying Wuu: Modifications made for xdi, v 1.0         */
/* 91/02/22     /f/osi/others/quipu/uips/pod/RCS/calls.c,v 7.2          */
/*              Isode Interim (6.8)                               	*/
/************************************************************************/

/* $Header: calls.c,v 1.8 92/09/01 16:33:34 sywuu Exp $ */

/*
 *    Original source: ISODE Interim Release 6.8
 *    Changes/additions by: Sze-Ying Wuu, Bellcore, <sywuu@thumper.bellcore.com>
 *
 *               Changes and Additions Copyright (c) 1991 Bellcore
 *                            All Rights Reserved
 *
 *    Permission is granted to copy, modify and redistribute this program
 *    in source or binary form provided that it is not sold for
 *    profit, and all copyright notices, permission and nonwarranty notices 
 *    remain intact.
 *
 *    Bellcore makes no warranty and accepts no liability for this program.
 * 
 *
 *                             ISODE NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */


#include <sys/types.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>

#include "quipu/util.h"
#include "quipu/common.h"
#include "quipu/entry.h"
#include "quipu/name.h"
#include "quipu/list.h"

#include "usr.dirent.h"
#include "tailor.h"
#include "sequence.h"
#include "filt.h"
#include "defs.h"
#include "util.h"

/* external routines */
void set_default_type(), make_friendly();
void quit();
void dn_print();
int rdn_print(), as_print();
char *TidyString();

/* external variables */
extern mailtype mailformat;
extern Attr_Sequence read_types, read_types2, oclass;
extern bool photo_on;

extern int sizelimit;
extern char *local_dit;
extern int  dsap_ad, main_dsa_id;
extern char bound;

/* global variables */
int typetoggled = 0;

Attr_Sequence get_sorted_attrs();

str_seq dnseq = NULLDS, backseq = NULLDS, showseq = NULLDS;
int entry_number, back_buf_num, dn_number, histlimit = 50;
bool read_all_flag = TRUE;
bool testing = FALSE;

char            goto_path[STRINGLEN];         
char            base_path[STRINGLEN];         /* current posisition */
char 		friendly_base_path[STRINGLEN];/* friendly name of base_path */
char            namestr[STRINGLEN];		
char            mvalue [STRINGLEN];	      /* name input */
char   		passwd[STRINGLEN];

unsigned int curr_filt = 0;		      /* index of filters */
unsigned int filt_num = 0;
unsigned int typeindx = 0;
char    *entrytype[MAXTYPES];

int default_num;
int *av_typeindx;
int *available_types[MAXTYPES];
char *levels[MAXTYPES];
int defaults[MAXTYPES];

DN user_name;

char * addobj = NULLCP;
Filter search_filter;
FILE *config_file;
char *file_names[MAXTYPES];
char dua_help_dir[SMALLSTRING];

/* get OID in string */
char *get_strioid(ptr)
register char *ptr;
{
  register char *end_ptr;

  while(*ptr == '"') ptr++;
  while(*ptr != '"') ptr++;

  while(*ptr > '9' || *ptr < '0') ptr++;

  end_ptr = ptr;
  while(*end_ptr != '\n') end_ptr++;

  *end_ptr = '\0';
  return ptr;
}

/* user tailor file configuration */
void user_tailor()
{
  char *part1, *part2;
  char *getenv(), *TidyString(), *SkipSpace();
  char *config_dir = "/.duaconfig",
       *type_dir = "/filterTypes",
       *readTypes = "/readTypes",
       *options = "/.xdirc",
       *quipurc = "/.quipurc",
       *typeDefaults = "/typeDefaults";

  char user_path[BUFSIZ], read_path[BUFSIZ], 
       type_path[BUFSIZ], options_path[BUFSIZ], quipurc_path[BUFSIZ], 
       type_defaults_path[BUFSIZ], dua_config_dir[BUFSIZ];

  char stroid_buf[BUFSIZ];
  char Read_in_Stuff[STRINGLEN];

  register char *p, *end;

  DIR *config_directory;
  struct dirent  *dir_ent;

  int count, n, num;
  int tempints[MAXTYPES];

  (void) strcpy(dua_config_dir, isodefile("xdi", 0));
  (void) strcpy(dua_help_dir, dua_config_dir);

  (void) strcat(dua_config_dir, "/duaconfig");
  (void) strcat(dua_help_dir, "/xdiHelpdir/");

  (void) strcpy (user_path, getenv("HOME"));
  (void) strcpy(read_path, user_path);
  (void) strcpy(type_path, user_path);
  (void) strcpy(type_defaults_path, user_path);
  (void) strcpy(options_path, user_path);
  (void) strcpy(quipurc_path, user_path);
  (void) strcat(options_path, options);
  (void) strcat(quipurc_path, quipurc);
  (void) strcat(read_path, config_dir);
  (void) strcat(read_path, readTypes);
  (void) strcat(type_path, config_dir);
  (void) strcat(type_path, type_dir);
  (void) strcat(type_defaults_path, config_dir);
  (void) strcat(type_defaults_path, typeDefaults);
	
  if (testing) {
    (void) strcpy(dua_help_dir, "./Xdi/xdiHelpdir/");
    (void) strcpy(type_path, "./Xdi/duaconfig/");
    (void) strcat(type_path, type_dir);
    if (!(config_directory = opendir(type_path))) {
      (void) fprintf(stderr, "File error! Test xdi should be run from source directory!\n");
      quit(1);
    }
  } else {
    if (!(config_directory = opendir(type_path))) {
      (void) strcpy(type_path, dua_config_dir);
      (void) strcat(type_path, type_dir);
      if(!(config_directory = opendir(type_path))) {
	(void) fprintf(stderr, "Can't find directory %s!\n", type_dir);
	quit(1);
      }
    }
  }

  rewinddir(config_directory);
  filt_num = 0;
  while(dir_ent = readdir(config_directory)) {
    if (!(strncmp(dir_ent->d_name, "Type_", 5))) {
      file_names[filt_num] = 
	(char *) malloc((unsigned int) 
			(strlen(dir_ent->d_name) + 
			strlen(type_path) + 2));
      (void) strcpy(file_names[filt_num], type_path);
      (void) strcat(file_names[filt_num], "/");
      (void) strcat(file_names[filt_num], dir_ent->d_name);
      filt_num++;
    }
  }
  (void) closedir(config_directory);
  
  if ((config_file = fopen (options_path, "r")) == 0)
     config_file = fopen (quipurc_path, "r");
  if (config_file) {
    while (fgets (Read_in_Stuff, STRINGLEN, config_file) != 0) {
      p = SkipSpace (Read_in_Stuff);
      if (( *p == '#') || (*p == '\0'))
	continue;  /* ignore comments and blanks */
      
      part1 = p;
      if ((part2 = index (p,':')) == NULLCP) 
	continue; /* ignore it */
      
      *part2++ = '\0';
      part2 = TidyString(part2);
      
      if ((lexequ(part1, "username") == 0) && *namestr == '\0')
	(void) strcpy (namestr, part2);
      else if ((lexequ(part1, "password") == 0) && *passwd == '\0') 
	(void) strcpy (passwd, part2);
      else if (lexequ(part1, "prefergreybook") == 0)
	mailformat = greybook;
      else if (lexequ (part1, "dsap") == 0)
	(void) tai_string (part2);
      else if (lexequ (part1, "isode") == 0) {
	char *split;
	if ((split = index (part2,' ')) != NULLCP) {
	  *split++ = 0;
	  (void) isodesetvar (part2, split, 0);
	}
      } else if (lexequ(part1, "service") == 0) 
	new_service (part2);
      else if (lexequ(part1, "history") == 0) {
	(void) sscanf(part2, "%d", &histlimit);
	if (histlimit < 1) histlimit = 1;
      } else if (lexequ(part1, "readnonleaf") == 0) {
	if (lexequ(part2, "TRUE") == 0) read_all_flag = TRUE;
	else read_all_flag = FALSE;
      }
    }
    isodexport (NULLCP);
    (void) fclose(config_file);
  }

  if (testing) {
    (void) strcpy(read_path, "./Xdi/duaconfig/");
    (void) strcat(read_path, readTypes);
    if (!(config_file = fopen(read_path, "r"))) {
      (void) fprintf(stderr, "File error! Test xdi must be run from source directory!\n");
      quit(1);
    }
  } else {
    if (!(config_file = fopen(read_path, "r"))) {
      (void) strcpy(read_path, dua_config_dir);
      (void) strcat(read_path, readTypes);
      if (!(config_file = fopen(read_path, "r"))) {
	(void) fprintf(stderr, "Can't find read file (%s)!\n", read_path);
	quit(1);
      }
    }
  }
  
  while(fgets(Read_in_Stuff, STRINGLEN, config_file) != 0) {
    (void) strcpy(stroid_buf, get_strioid(Read_in_Stuff));
    if (*stroid_buf) {
      if (!read_types)
	read_types = as_comp_new(AttrT_new(stroid_buf), NULLAV, NULLACL_INFO);
      else {
	read_types2 = as_comp_new(AttrT_new(stroid_buf), NULLAV, NULLACL_INFO);
	read_types = as_merge(read_types, read_types2);
      }
    }
  }
  (void) fclose(config_file);

  for (curr_filt = 0; curr_filt < filt_num; curr_filt++) {
    if (!(config_file = fopen(file_names[curr_filt], "r"))) {
      (void) fprintf(stderr, "Can't find file %s!\n", file_names[curr_filt]);
      quit(1);
    }
    
    while (!yyparse());

    (void) fclose(config_file);
  }

  entrytype[curr_filt] = NULLCP;
  for (count = 0; count < filt_num; count++)
    free(file_names[count]);

  if (testing) {
    (void) strcpy(type_defaults_path, "./Xdi/duaconfig/");
    (void) strcat(type_defaults_path, typeDefaults);
    if (!(config_file = fopen(type_defaults_path, "r"))) {
      (void) fprintf(stderr, "File error! Test xdi must be run from source directory.\n");
      quit(1);
    }
  } else {
    if (!(config_file = fopen(type_defaults_path, "r"))) {
      (void) strcpy(type_defaults_path, dua_config_dir);
      (void) strcat(type_defaults_path, typeDefaults);
      if (!(config_file = fopen(type_defaults_path, "r"))) {
	(void) fprintf(stderr, "File error! Can't find typeDefaults file.\n");
	quit(1);
      }
    }
  }

  default_num = 0;
  while (fgets (Read_in_Stuff, STRINGLEN, config_file) != 0) {
    p = SkipSpace(Read_in_Stuff);
    if (( *p == '#') || (*p == '\0'))
      continue;
    
    part1 = p;
    if ((part2 = index (p,':')) == NULLCP)
      continue;
    
    end = part2 - 1;
    while (isspace(*end)) end--;
    *++end = '\0';

    *part2++ = '\0';

    while (isspace(*part2)) part2++;
    end = part2;

    while (!isspace(*end) && *end != ',' && *end != ':') end++;

    count = 0;
    while (*part2 != ':') {
      n = 0;
      while (n < filt_num && strncmp(entrytype[n], 
				     part2, (int) (end - part2))) n++;
      
      if (n == filt_num) {
	(void) fprintf(stderr, "Parsing error in typeDefaults file!");
	quit(1);
      } else {
	tempints[count] = n;
	count++;
	part2 = end;
	while (!isalpha(*part2) && *part2 != ':' && part2 != '\0') part2++;

	if (*part2 == '\0') {
	  (void) fprintf(stderr, "Parsing error in typeDefaults file!");
	  quit(1);
	} 

	if (*part2 != ':') {
	  while (!isalpha(*part2)) part2++;
	  end = part2;
	  while (!isspace(*end) && *end != ',' && 
		 *end != ':' && *end != '\0') end++;
	  if (*end == '\0') {
	    (void) fprintf(stderr, "Parsing error in typeDefaults file!");
	    quit(1);
	  }
	} else end = part2;
      }
    }

    if (*end == ':') {
      while(isspace(*++end));
      p = end;
      while(!isspace(*++end));
      *end = '\0';

      n = 0;
      while (n < filt_num && strcmp(entrytype[n], p)) n++;
      
      if (n == filt_num) {
	(void) fprintf(stderr, "Parsing error in typeDefaults file!");
	quit(1);
      } else {
	num = 0;
	while (num < count && n != tempints[num]) num++;
	if (num == count) {
	  (void) fprintf(stderr, "Parsing error in typeDefaults file!");
	  quit(1);
	}
      }

      defaults[default_num] = n;

      levels[default_num] = malloc((unsigned int) (strlen(part1) + 1));
      (void) strcpy(levels[default_num], part1);
      available_types[default_num] = 
	(int *) malloc((unsigned int) (sizeof(int) * (count+1)));

      for (n = 0; n < count; n++)
	available_types[default_num][n] = tempints[n];
      
      available_types[default_num][n] = -1;
      default_num++;
    }
  }

  (void) fclose(config_file);
}

/* set default entry type for search based on current position */
void set_default_type()
{
  int count;
  register DN base_name;
  DN d_name;

  if (*base_path != '\0') {
    base_name = d_name = str2dn(base_path);

    while (base_name && base_name->dn_parent) base_name = base_name->dn_parent;
    for (count = 0; 
	 count < default_num && base_name && 
	 strcmp(levels[count], 
		base_name->dn_rdn->rdn_at->
		oa_ot.ot_stroid);
	 count++);
    dn_free(d_name);

    if (count < default_num) {
      av_typeindx = available_types[count];
      typeindx = defaults[count];
    } else {
      av_typeindx = available_types[0];
      typeindx = defaults[0];
    }
  } else {
    for(count = 0; count < default_num && strcmp(levels[count], "@"); count++);
    if (count < default_num) {
      av_typeindx = available_types[count];
      typeindx = defaults[count];
    } else {
      av_typeindx = available_types[0];
      typeindx = defaults[0];
    }
  }
  typetoggled = 0;
}

/* convert DN to string */
void dn2buf (ptr,cptr)
caddr_t ptr;
char * cptr;
{
  PS ps;
  char buffer [RESBUF];
  
  if((ps = ps_alloc(str_open)) == NULLPS) return ;

  if(str_setup(ps, buffer, RESBUF, 1) == NOTOK) return ;
  
  dn_print(ps, (DN) ptr, EDBOUT);
  *ps->ps_ptr = 0;
  
  ps_free (ps);

  (void) strcpy(cptr, buffer);
}


/* convert DN to user-friendly name */
void make_friendly(fstr, str)
char *fstr, *str;
{
  register char *end, *start;
  char save;
  
  *fstr = '\0';
  if (*str == '\0') {
    *fstr = '\0';
    return;
  }

  end = str;
  while (*end != '\0')  end++;

  while (1) {
     while (!(isalnum(*end))) end--;
     start = end++;
     while (*start != '=') start--;
     start++;
     save = *end;
     *end = '\0';
     (void) strcat(fstr, start);
     *end = save;

     while (*start != '@' && *start != '%' && start > str) start--;
     if (*start == '@') 
        (void) strcat(fstr, ", ");
     else if (*start == '%') 
        (void) strcat(fstr, " + ");
     else if (start <= str) 
        break;
     end = start;
  }
}
  
/* convert RDN to user-friendly name */
make_friendly_rdn(friendly, object, base)
char *friendly;
char *object, *base;
{
  register char *front;
  int count;

  *friendly = '\0';

  front = base;
  count = 0;
  if (showseq != backseq) {
    while (*front != '\0') {
      if (*front == '=') count++; 
      front++;
    }
  } else
    count = 0;

  front = object;

  while (count && *front != '\0') {
    if (*front == '=') count--; 
    front++;
  }
  
  if (front != object) {
    while (*front != '\0' && *front != '@') front++;
    while (!isalpha(*front) && *front != '\0') front++;
  }
  make_friendly(friendly, front);
}

/* change directory position */
goto_addr()
{
  char *str;
  int count = 0;
  void add_to_history();

  set_default_type();

  make_friendly(friendly_base_path, base_path);

  str = get_from_seq(count+1, backseq);
  while (count < back_buf_num && strcmp(str, base_path)){
    count++;
    str = get_from_seq(count+1, backseq);
  }	

  if (count == back_buf_num) {
    add_seq(&backseq, base_path);
    back_buf_num++;
    add_to_history(back_buf_num);
  }
}

/* clearing DN sequence buffer */
clear_dnseq()
{
  free_seq(dnseq);
  dnseq = NULLDS;
  dn_number = 0;
}

/* checking if a DN corresponds to a leaf node */
int isleafnode(name)
char *name;
{
  struct ds_list_arg list_arg;
  struct ds_list_result   list_result;
  struct list_cache *cached_list;

  struct ds_read_arg read_arg;
  struct ds_read_result   read_result;
  Entry read_entry;
  Attr_Sequence object_class;

  struct DSError          list_error, read_error;

  char entry_str[STRINGLEN];

  if (get_default_service (&read_arg.rda_common) != 0) return(1);
  read_arg.rda_common.ca_servicecontrol.svc_options = SVC_OPT_PREFERCHAIN;
  read_arg.rda_eis.eis_allattributes = FALSE;
  read_arg.rda_eis.eis_infotypes = EIS_ATTRIBUTESANDVALUES;
  read_arg.rda_eis.eis_select = oclass;

  read_arg.rda_object = (*name? str2dn(name): NULLDN);

  if ((read_entry = local_find_entry(read_arg.rda_object, FALSE))
      != NULLENTRY &&
      read_entry->e_data != E_TYPE_CONSTRUCTOR) {
    object_class = get_sorted_attrs(read_entry->e_attributes,
				    oclass);
    entry2str((caddr_t) object_class, entry_str, STRINGLEN);
    as_free(object_class);
    if (issubstr(entry_str, "NonLeaf")) return(0);
  } else if (ds_read(&read_arg, &read_error, &read_result) == DS_OK) {
    ds_error_free(&read_error);
    if (read_result.rdr_entry.ent_attr == NULLATTR) return(0);
    entry2str((caddr_t) read_result.rdr_entry.ent_attr, entry_str, STRINGLEN);
    entryinfo_comp_free(&read_result.rdr_entry, 0);
    if (issubstr(entry_str, "NonLeaf")) return(0);
  } else
    ds_error_free(&read_error);


  dn_free(read_arg.rda_object);

  if (get_default_service (&list_arg.lsa_common) != 0) return(1);
  list_arg.lsa_common.ca_servicecontrol.svc_sizelimit = 1;
  list_arg.lsa_common.ca_servicecontrol.svc_options = SVC_OPT_PREFERCHAIN;
  list_arg.lsa_object = (*name? str2dn(name): NULLDN);

  if ((cached_list = find_list_cache(list_arg.lsa_object, 1)) != NULL) {
    if (cached_list->list_sub_top) return(0);
    else return(1);
  } else if (ds_list (&list_arg, &list_error, &list_result) == DS_OK) {
    cache_list(list_result.lsr_subordinates, 0, list_arg.lsa_object, 1);
    dn_free(list_arg.lsa_object);
    if (list_result.lsr_subordinates) return(0);
    else return(1);
  } else {
    dn_free(list_arg.lsa_object);
    ds_error_free(&list_error);
    return (1);
  }
}

/* get surname of a RDN */
char *GetSurname(name)
char *name;
{
  char *start, *end; 
  char sn[STRINGLEN], save; 
  int n;

  start = name;
  n = indexstring(start, "sn"); 
  if (n == -1) n = indexstring(start, "cn");

  start = start + n;
  while (*start != '=') start++;
  end = ++start;
  while (1) {
    while (isalpha(*end) || (*end == '-') || (*end == '.')) end++;
    if ((*end == '\0') || (*end != ' ')) break;
    start = ++end;
  }

  save = *end;
  *end = '\0';
  strcpy(sn, start);
  *end = save;
  return sn;
}

/* get last RDN */
char *GetWholeRelName(name)
register char *name;
{
  while (*name!= '\0') name++;
  while (*name != '=') name--;
  while (!isalpha(*name)) name++;
  return name;
}

/* sort a list of DNs */
str_seq SortList(list)
str_seq list;
{
  register str_seq currEntry, lastSortedEntry , currSortedEntry; 
  str_seq sortedList;
  char sortedName[STRINGLEN], *currName[STRINGLEN];
  register DN curr_dn;
  register RDN curr_rdn;
  DN dn;
  int cn_exist = FALSE;

  if (!list) return 0;

  currEntry = list;
  sortedList = 0;

  while (currEntry) {
    dn = curr_dn = str2dn(currEntry->dname);

    while (curr_dn && curr_dn->dn_parent != NULLDN) 
      curr_dn = curr_dn->dn_parent;
    
    curr_rdn = curr_dn->dn_rdn;
    cn_exist = FALSE;
    while (curr_rdn != NULLRDN) {
      if (!strcmp(curr_rdn->rdn_at->oa_ot.ot_stroid, "2.5.4.3")) {
	cn_exist = TRUE;
        break;
      }
      curr_rdn = curr_rdn->rdn_next;
    }
       
    if (cn_exist)
      strcpy(currName, GetSurname(currEntry->dname));
    else 
      strcpy(currName, GetWholeRelName(currEntry->dname));

    dn_free(dn);

    if (!sortedList) {
      sortedList = currEntry;
      currEntry = currEntry->next;
      sortedList->next = 0;
    } else {
      lastSortedEntry = 0;
      currSortedEntry = sortedList;

      while (currSortedEntry != 0) {
	dn = curr_dn = str2dn(currSortedEntry->dname);
	
	while (curr_dn && curr_dn->dn_parent != NULLDN) 
	  curr_dn = curr_dn->dn_parent;
	
        curr_rdn = curr_dn->dn_rdn;
        cn_exist = FALSE;
        while (curr_rdn != NULLRDN) {
          if (!strcmp(curr_rdn->rdn_at->oa_ot.ot_stroid, "2.5.4.3")) {
	     cn_exist = TRUE;
             break;
          }
          curr_rdn = curr_rdn->rdn_next;
        }
       
        if (cn_exist)
	  strcpy(sortedName, GetSurname(currSortedEntry->dname));
	else 
	  strcpy(sortedName, GetWholeRelName(currSortedEntry->dname));
	
	dn_free(dn);

	if (strcmp(currName, sortedName) <= 0) {
	  if (lastSortedEntry) {
	    lastSortedEntry->next = currEntry;
	    currEntry = currEntry->next;
	    lastSortedEntry->next->next = currSortedEntry;
	  } else {
	    sortedList = currEntry;
	    currEntry = currEntry->next;
	    sortedList->next = currSortedEntry;
	  }
	  currSortedEntry = 0;
	} else {
	  lastSortedEntry = currSortedEntry;
	  currSortedEntry = currSortedEntry->next;
	  if (!currSortedEntry) {
	    lastSortedEntry->next = currEntry;
	    currEntry = currEntry->next;
	    lastSortedEntry->next->next = 0;
	    currSortedEntry = 0;
	  }
	}
      }
    }
  }
  return sortedList;
}
