/************************************************************************/
/* search.c: This file contains routines for Search Dap operations      */
/*                                                                      */
/* Modification Log:                                                    */
/*                                                                      */
/* 91/05/21     Sze-Ying Wuu: Additions/changes made for xdi, v 1.0     */
/* 91/02/22     /f/osi/others/quipu/uips/pod/RCS/search.c,v 7.2         */
/*              Isode Interim (6.8)                               	*/
/************************************************************************/

/* $Header: search.c,v 1.8 92/09/01 16:35:11 sywuu Exp $ */

/*
 *    Original source: ISODE Interim Release 6.8
 *    Changes/additions by: Sze-Ying Wuu, Bellcore, <sywuu@thumper.bellcore.com>
 *
 *               Changes and Additions Copyright (c) 1991 Bellcore
 *                            All Rights Reserved
 *
 *    Permission is granted to copy, modify and redistribute this program
 *    in source or binary form provided that it is not sold for
 *    profit, and all copyright notices, permission and nonwarranty notices 
 *    remain intact.
 *
 *    Bellcore makes no warranty and accepts no liability for this program.
 * 
 *
 *                             ISODE NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */

/* header files */
#include <sys/types.h>
#include "quipu/util.h"
#include "quipu/common.h"
#include "quipu/entry.h"
#include "quipu/name.h"
#include "quipu/ufn.h"

#include "sequence.h"
#include "filt.h"
#include "select.h"
#include "defs.h"

/* external variables */
extern str_seq dnseq, backseq, showseq;
extern int entry_number, back_buf_num, dn_number;
extern int sizelimit;
extern unsigned int typeindx;
extern ToggleList dua_tlist;
extern ToggleList sel_tlist;
extern KeyList key_list;
extern PS	opt;

#ifndef NO_STATS
extern LLog    *log_dsap;
#endif

extern char goto_path[], base_path[], friendly_base_path[], mvalue[];
extern char *linking_dit;

extern str_seq SortList();
extern dsEnqError list_start(), geterr();
extern void dn2buf();
extern void displayError();
extern char *issubstr();

/* global variables */
int	chase_flag;

/************************************************************************/
/* srch_start: invokes DAP search operation 				*/
/*       filt: filter for the search operation 				*/
/************************************************************************/

dsEnqError srch_start(filt)
Filter	filt;
{
  struct ds_search_arg search_arg;
  struct ds_search_result result;
  struct DSError          error;
  dsEnqError return_error;
  DN curr_rdn;
  int i, srch_err;
  DNS newdns, res = NULLDNS;

  
  if (filt == NULLFILTER) return duaerror;

  if (get_default_service (&search_arg.sra_common) != 0) 
    return nothingfound;

  if (FullList)
     search_arg.sra_common.ca_servicecontrol.svc_sizelimit = SVC_NOSIZELIMIT;

  search_arg.sra_common.ca_servicecontrol.svc_options = SVC_OPT_PREFERCHAIN;

  curr_rdn = search_arg.sra_baseobject = (strcmp(friendly_base_path, "The World")?
					  str2dn (base_path):
					  NULLDN);

  search_arg.sra_eis.eis_allattributes = TRUE;
  search_arg.sra_eis.eis_infotypes = EIS_ATTRIBUTESANDVALUES;
  search_arg.sra_eis.eis_select = 0;
  search_arg.sra_searchaliases = TRUE;
  search_arg.sra_subset = SRA_ONELEVEL;

  while (curr_rdn != NULLDN) {
    if (!strcmp(curr_rdn->dn_rdn->rdn_at->oa_ot.ot_stroid, 
		"2.5.4.10")) {
      search_arg.sra_subset = SRA_WHOLESUBTREE;
      break;
    }
    curr_rdn = curr_rdn->dn_parent;
  }

  search_arg.sra_filter = filt;
/*
#ifndef NO_STATS
  if (filt)
     LLOG (log_dsap, LLOG_NOTICE, ("search +%s, extent %d, val %s",
                                   base_path,search_arg.sra_subset, filt));
  else
     LLOG (log_dsap, LLOG_NOTICE, ("search +%s, extent %d, val %s",
                                   base_path,search_arg.sra_subset, mvalue));
#endif
*/

  srch_err = do_search(&search_arg, &error, &result);
  if (srch_err != DS_OK) {
    /* deal with error */
    free_seq(dnseq);
    dnseq = NULLDS;
    dn_number = 0;
    return_error = geterr(error.dse_type);
    log_ds_error(&error);
    ds_error_free(&error);
  } else {
    correlate_search_results (&result);
    dn_number = 0;

    if (result.CSR_entries != NULLENTRYINFO) {
      register EntryInfo *ptr;

      return_error = Okay;
      free_seq(dnseq);
      dnseq = NULLDS;
      dn_number = 0;

      for (ptr = result.CSR_entries; ptr != NULLENTRYINFO; ptr=ptr->ent_next) {
		cache_entry(ptr, FALSE, TRUE);
                newdns = dn_seq_alloc();
                newdns->dns_next = res;
                newdns->dns_dn = dn_cpy (ptr->ent_dn);
                res = newdns;
      }

      if (AttrSrch) {
	for (i = 0; i < key_list.n_kitems; i++) {
	    if (key_list.kitems[i].val) 
		DnSelect(key_list.kitems[i].val, &res);
        }
      }
      else
        DnSelect (mvalue,&res);
      while (res) {
        dn2buf((caddr_t) res->dns_dn, goto_path);
        dn_number++;
        add_seq(&dnseq, goto_path);
        res = res->dns_next;
      }

      if (dn_number) 
	dnseq = SortList(dnseq);
      else 
	return_error = nothingfound;

    } else if (result.CSR_limitproblem == LSR_NOLIMITPROBLEM) {
      free_seq(dnseq);
      dnseq = NULLDS;
      dn_number = 0;
      return_error = nothingfound;
    }

    if(result.CSR_limitproblem != LSR_NOLIMITPROBLEM) {
      switch (result.CSR_limitproblem) {
      case LSR_TIMELIMITEXCEEDED:
	if (dn_number > 0) return_error = timelimit_w_partial;
	else {
	  free_seq(dnseq);
	  dnseq = NULLDS;
	  return_error = timelimit;
	}
	break;
      case LSR_SIZELIMITEXCEEDED:
	return_error = listsizelimit;
	break;
      case LSR_ADMINSIZEEXCEEDED:
	if (dn_number > 0) return_error = adminlimit_w_partial;
	else {
	  free_seq(dnseq);
	  dnseq = NULLDS;
	  return_error = adminlimit;
	}
	break;
      }
      entryinfo_free(result.CSR_entries, 0);
    }
  }
  entry_number = dn_number;
  if (search_arg.sra_baseobject) dn_free(search_arg.sra_baseobject);
  ds_error_free(&error);
  return return_error;
}

/* search by user-friendly name */
dsEnqError ufn_srch()
{
  extern Filter make_filter(), get_filter();
  struct dn_seq *dns = NULL;
  int n;
  char *vec[20];
  DN base;

  getvec(vec, &n, mvalue);
  
  if (ufn_init() == FALSE) {
	return duaerror;
  }

  base = (strcmp(friendly_base_path, "The World")? str2dn (base_path): NULLDN);
  if (!do_ufn_match(n, vec, &dns, NULLEL)) {
	return nothingfound;
  }

  free_seq(dnseq);
  dnseq = NULLDS;
  dn_number = 0;

  while (dns) {
      dn2buf((caddr_t) dns->dns_dn, goto_path);
      dn_number++;
      add_seq(&dnseq, goto_path);
      dns = dns->dns_next;
  }

  if (dn_number) dnseq = SortList(dnseq);
  else return nothingfound;

  entry_number = dn_number;
  return Okay;
}

/* Search DAP */
do_search(search_arg, error, result)
struct ds_search_arg	*search_arg;
struct DSError		*error;
struct ds_search_result	*result;
{
	int	n;

	/* Strong authentication */
	if (search_arg->sra_common.ca_security != (struct security_parms *) 0) {
/*
		struct signature *sign_operation();
*/
		int encode_DAS_SearchArgumentData();	
/*
		search_arg->sra_common.ca_sig =
			sign_operation((caddr_t)search_arg, encode_DAS_SearchArgumentData);
*/
		search_arg->sra_common.ca_sig = (struct signature *) NULL;

	}

       /* syw: synchronous multiple associations with multiple DSAs */
        while ((n = ds_search (search_arg, error, result)) != DS_OK) {
                if (srch_error (opt, error) == 0)
                        return(n);
                search_arg->sra_baseobject = error->ERR_REFERRAL.DSE_ref_candidates->cr_name;
        }

        correlate_search_results (result);     /* syw: correlate results from
multiple associations */
	return (DS_OK);
}


getvec(vec, n, str)
char **vec;
int *n;
char *str;
{
  char *p, *s;
  int i = 0;

  p = str;

  while (*p != '\0') {
	s = index(p, ',');
	if (!s) {
		vec[i] = strdup(p);
		i++;
		break;
	}
	
	*s = '\0';
	vec[i] = strdup(p);
	i++;
	*s = ',';
	p = s + 1;
  }

  *n = i;
}

	
	
	

  

